#!/usr/bin/env bash
# -*- coding: utf-8 -*-
# coding: utf-8
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
source /scripts/basic_vars
set -o nounset
set -o pipefail
set -o errexit
exec 1> >(exec systemd-cat -p info -t apply_customization_apps) 2>&1
set -o xtrace
serial=$(cat /config/.serial-number)
hwtype=$(cat /config/.hwtype)
if [ -z "${serial}" -o -z "${hwtype}" ]; then
  logger -t apply_customization_apps "Unknown serial --${serial}-- or hwtype --${hwtype}--!"
  exit 1
fi
cd /tmp
APPS_SEARCH_PATH=""
if [ -d ${customization_dir}/apps/${serial} ]; then
  APPS_SEARCH_PATH=${APPS_SEARCH_PATH}" ${customization_dir}/apps/${serial}/"
elif [ -d ${customization_dir}/apps/${hwtype} ]; then
  APPS_SEARCH_PATH=${APPS_SEARCH_PATH}" ${customization_dir}/apps/${hwtype}/"
elif [ -d ${customization_dir}/apps/ANY ]; then
  APPS_SEARCH_PATH=${APPS_SEARCH_PATH}" ${customization_dir}/apps/ANY/"
fi
if [ -z "${APPS_SEARCH_PATH}" ]; then
  logger -p daemon.warn -t apply_customization "No customization directory !"
  exit 0
fi
APPS=$(find ${APPS_SEARCH_PATH} -name "*.tar.gz" -print -exec cp -f {} /home/admin/user_files \; | tr -s '\n' ' ')
if [ -n "${APPS}" ]; then
  for app in ${APPS}; do
    if [ -f /home/admin/user_files/$(basename ${app}) ]; then
      FAKEAPPFULL=$(basename ${app})
      FAKEAPPNAME=${FAKEAPPFULL%%.tar.gz}
      rm -rf /var/tmp/${FAKEAPPNAME}
      mkdir /var/tmp/${FAKEAPPNAME}
      tar xzf /home/admin/user_files/$(basename ${app}) -C /var/tmp/${FAKEAPPNAME}
      APPJSON=$(find /var/tmp/${FAKEAPPNAME} -name application.json)
      APPNAME=$(/usr/bin/jq '.name' ${APPJSON} | tr -d '"' |tr '[:upper:]' '[:lower:]')
      cd /var/tmp/${FAKEAPPNAME}
      rm -f /var/tmp/${APPNAME}.tar.gz
      cd $(dirname ${APPJSON})/../
      tar cf /var/tmp/${APPNAME}.tar *
      mkdir -p /root/apps/${APPNAME}
      cd /tmp
      tar xf /var/tmp/${APPNAME}.tar -C /root/apps/${APPNAME}
      rm -rf /var/tmp/${FAKEAPPNAME} /var/tmp/${APPNAME}.tar
      cp /root/apps/${APPNAME}/${APPNAME}.service /lib/systemd/system
      chmod 644 /lib/systemd/system/${APPNAME}.service
      systemctl daemon-reload
      APPDIR=/root/apps/${APPNAME}
      CONFIG=$(/usr/bin/jq '.config' ${APPDIR}/config/application.json | tr -d '"')
      AUTOSTART=$(/usr/bin/jq '.autostart' ${APPDIR}/config/${CONFIG} | tr -d '"')
      if [ -n "$(echo ${APPNAME} | grep --ignore-case clavister)" ] \
         && [ -z "${is_native_app_clavister_enabled_in_license}" ]; then
        logger -p daemon.warn -t apply_customization "Native application Clavister not allowed by a valid license !"
      elif [ -n "$(echo ${APPNAME} | grep --ignore-case mikrotik)" ] \
           && [ -z "${is_native_app_mikrotik_enabled_in_license}" ]; then
        logger -p daemon.warn -t apply_customization "Native application MikroTik not allowed by a valid license !"
      elif [ "true" = "${AUTOSTART}" ]; then
        systemctl enable ${APPNAME}.service 2>/dev/null
      fi
    fi
  done
fi

exit 0
