#!/bin/bash
#/ Usage: helm_prepare.sh [option]
#/
#/     create                   creates environment to run Helm in chroot jail
#/     clear                    deletes created environment
#/
#/ Helm prepare shell script creates environment to run Helm in chroot jail.
#/ This script is used by helm_cli.sh

HELM_JAIL_DIR='/var/tmp/helm_bash_jail/'
FOLDERS=('bin' 'config' 'etc' 'lib' 'lib64' 'charts' 'templates' 'binaries')
# Get necessary libraries with 'ldd $(which <binary>)'
FILES=('/bin/bash bin/' '/bin/helm binaries/' '/bin/tar binaries/' '/bin/grep binaries/' '/bin/head binaries/' '/bin/rm binaries/'\
    '-r /etc/ssl/ etc/' '/root/.kube/config config/config' '/lib/x86_64-linux-gnu/libtinfo.so.6 lib/' \
    '/lib/x86_64-linux-gnu/libc.so.6 lib/' '/lib64/ld-linux-x86-64.so.2 lib64/' \
    '/scripts/cli/tvesh/helm_cli.sh etc/bash.bashrc' '/lib/x86_64-linux-gnu/libacl.so.1 lib/' \
    '/lib/x86_64-linux-gnu/libselinux.so.1 lib/' '/lib/x86_64-linux-gnu/libpcre2-8.so.0 lib/' \
    '/lib64/ld-linux-x86-64.so.2 lib64/' '/lib/x86_64-linux-gnu/libpcre.so.3 lib/' '/etc/resolv.conf etc/')

function create_helm_jail() {
    if [[ -d $HELM_JAIL_DIR ]]; then
        clear_helm_jail
    fi

    # Create folder structure
    mkdir $HELM_JAIL_DIR
    pushd $HELM_JAIL_DIR > /dev/null

    for idx in ${!FOLDERS[@]}; do
        mkdir ${FOLDERS[$idx]}
    done

    # Add binaries and certificates
    # TODO: Get bash libraries with 'ldd /bin/bash'
    for idx in ${!FILES[@]}; do
        cp ${FILES[idx]}
        if [[ $? -ne 0 ]]; then
            echo "Failed to copy ${FILES[idx]}."
            exit 1
        fi
    done

    # Mount /home/admin/user_files into charts/ local directory
    # Mount /kubernetes/helm into templates/ local directory
    if [[ -d /home/admin/user_files && -d /kubernetes/helm ]]; then
        mount --bind /home/admin/user_files/ charts/
        mount --bind /kubernetes/helm/ templates/
    fi

    popd > /dev/null
}

function clear_helm_jail () {
    pushd $HELM_JAIL_DIR > /dev/null
    # These commands can disrupt a running process, cause data loss
    # or corrupt open files.
    if [ -d 'charts/' ]; then
        umount -l -q charts/
        umount -f -q charts/
    fi
    if [ -d 'templates/' ]; then
        umount -l -q templates/
        umount -f -q templates/
    fi
    popd > /dev/null
    rm -rf $HELM_JAIL_DIR
}

case "$1" in
  create)
    create_helm_jail
    ;;
  clear)
    clear_helm_jail
    ;;
  *)
    echo "Unknown operation"
    exit 1
    ;;
esac

exit 0
