#!/bin/bash
# -*- coding: utf-8 -*-
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
#################################################################
### SCRIPT TO SET OPENSTACK CONTROL ADDRESS
#################################################################
set -e # EXIT IMMEDIATELY UPON ERROR!
shopt -s expand_aliases
### SOURCE SAME BASIC VARS NEEDED
source /scripts/basic_vars
source /root/admin-openrc.sh
### SOURCE SAME BASIC FUNCTIONS NEEDED
source /scripts/basic_functions
if [ "$LOG_IN_FILE" -eq 1 ]; then
  enable_logging /var/log/homecalling
fi

source /scripts/ipv6_functions

new_control_ip=$1
rebind_flag=$2
bind_interface=$(sysrepocfg -X -x /ucpe:config/kubernetes:kubernetes/bind-interface -f xml -d running | grep bind-interface | cut -d '>' -f 2 | cut -d '<' -f1 || true)
cbox_state_legacy=$(sysrepocfg -X -x /ucpe:config/openstack:openstack/cbox/enabled -f xml -d running | grep enabled | cut -d '>' -f 2 | cut -d '<' -f1 || true)
cbox_state=$(sysrepocfg -X -x /ucpe:config/kubernetes:kubernetes/cbox/enabled -f xml -d running | grep enabled | cut -d '>' -f 2 | cut -d '<' -f1 || true)
bind_interface_proto=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='$bind_interface']/bootproto" -f xml -d running | grep bootproto | cut -d '>' -f 2 | cut -d '<' -f1 || true)
hostname=$(sysrepocfg -X -x "/ucpe:config/system:system/hostname" -f xml -d running | grep hostname | cut -d '>' -f 2 | cut -d '<' -f1 || true)

ipv_changed=false

if [[ -f "$ipv6_flag_file" ]]; then
  ipv_changed=true
fi

if [ -z "$new_control_ip" ]; then
  if [ -n "$bind_interface_proto" ] && [ "none" == "$bind_interface_proto" ] ; then
    # Try IPv6 First
    new_control_ip=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='$bind_interface']/ipv6-prefix" -f xml -d running | grep ipv6-prefix | cut -d '>' -f 2 | cut -d '/' -f1 || true)
    if [[ -z $new_control_ip ]]; then
      # Get IPv4 then
      new_control_ip=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='$bind_interface']/ipv4-prefix" -f xml -d running | grep ipv4-prefix | cut -d '>' -f 2 | cut -d '/' -f1 || true)
    fi
  else
    if [ -d "/sys/class/net/br-$bind_interface" ]; then
      bind_interface=br-$bind_interface
      # Try IPv6 First
      new_control_ip=$(ifconfig br-$bind_interface | grep "inet6 " | grep -v -e "fe80:" | awk '{print $2}' || true)
      if [[ $new_control_ip == "" ]]; then
        new_control_ip=$(ifconfig br-$bind_interface | grep "inet " | awk '{print $2}' || true)
      fi
    fi
    if [ -d "/sys/class/net/$bind_interface" ]; then
      # This is for tun interfaces
      if [ -z "${new_control_ip}" ]  || [ "${new_control_ip}"=="" ]; then
        # Try IPv6 First
        new_control_ip=$(ifconfig $bind_interface | grep "inet6 " | grep -v -e "fe80:" | awk '{print $2}' || true)
        if [[ $new_control_ip == "" ]]; then
          new_control_ip=$(ifconfig $bind_interface | grep "inet " | awk '{print $2}' || true)
        fi
      fi
    fi
  fi
fi

if [ -n "$new_control_ip" ]; then

  new_ip=$new_control_ip

  # If running local controller most of services can be bound to 127.0.0.1 / ::1 instead
  if [ "true" == "$cbox_state" ] || [ "true" == "$cbox_state_legacy" ]; then
    # Correct loopback for IPv4 or IPv6
    if [[ "$new_control_ip" == *":"* ]]; then
      new_ip="::1"
    else
      new_ip="127.0.0.1"
    fi
  fi

  # Until the setting of openstack bind uses separate flow, not only from subscriber, such functionality should be present on several places
  if [[ "$rebind_flag" == "force_rebind" ]]; then

    if [[ "$new_control_ip" == *":"* ]]; then
        
        mv -f /etc/hosts /etc/hosts.tmpsro || true
        sed -i "s/^127\.0\.0\.1\s.*$/127\.0\.0\.1\ \ \ \ ipv4-localhost/" /etc/hosts.tmpsro || true
        sed -i "s/^::1\s.*$/::1\ \ \ \ localhost\ ip6-localhost\ ip6-loopback/" /etc/hosts.tmpsro || true
        mv -f /etc/hosts.tmpsro /etc/hosts || true

      # IPv6 Related settings
      # Create flag file
      touch "$ipv6_flag_file" || true

    else
      # IPv4 Related settings
      #Remove flag file
      rm -f "$ipv6_flag_file" || true
      
        mv -f /etc/hosts /etc/hosts.tmpsro || true
        sed -i "s/^127\.0\.0\.1\s.*$/127\.0\.0\.1\ \ \ \ localhost/" /etc/hosts.tmpsro || true
        sed -i "s/^::1\s.*$/::1\ \ \ \ ip6-localhost\ ip6-loopback/" /etc/hosts.tmpsro || true
        mv -f /etc/hosts.tmpsro /etc/hosts || true
    fi
  fi

  # This file will be executed in result of both IPv4 and IPv6 events. So do update of the IP only when
  # the new IP is with the same family with already bound
  if [[ "$ipv_changed" == "true" && "$new_control_ip" == *":"* ]] || [[ "$ipv_changed" != "true" && "$new_control_ip" != *":"* ]]; then

    # Beautify URL if IPv6
    if [[ "$new_control_ip" == *":"* ]]; then
      new_control_ip_url="[$new_control_ip]"
    else
      new_control_ip_url="$new_control_ip"
    fi

    # Beautify URL if IPv6
    if [[ "$new_ip" == *":"* ]]; then
      new_ip_url="[$new_ip]"
    else
      new_ip_url="$new_ip"
    fi

    echo "new_ip: $new_ip; new_control_ip: $new_control_ip" | systemd-cat

    if [ ! -f "/config/central-box" ]; then
      #Do not do this if we are in central controller (central-cbox)
      :
    fi
    ## cloud in a box, set controller IP
    if [ "true" == "$cbox_state" ] || [ "true" == "$cbox_state_legacy" ]; then

      # admin-openrc.env
      sed -i "s/^OS_AUTH_URL=.*/OS_AUTH_URL=https:\/\/${new_control_ip_url}:35357\/v3/" /root/admin-openrc.env || true
      # /etc/hosts
      mv -f /etc/hosts /etc/hosts.tmpso
      sed -i "/controller$/d" /etc/hosts.tmpso || true
      if [ -n "$hostname" ]; then
        sed -i "/$hostname\$/d" /etc/hosts.tmpso || true
        echo "$new_control_ip $hostname" >> /etc/hosts.tmpso || true
      fi
      mv -f /etc/hosts.tmpso /etc/hosts

    fi
  fi
fi

#
/bin/sync || true
if [ $(systemctl is-active isc-dhcp-relay.service) == "active" ]; then
  systemctl restart --no-block isc-dhcp-relay || true
fi
#
exit 0
